@@ -1,8 +1,8 @@
 from pipetools.utils import foreach
 
-__version__ = VERSION = 0, 1, 8
+__version__ = VERSION = 0, 1, 9
 
 __versionstr__ = VERSION > foreach(str) | '.'.join
 
-from pipetools.main import pipe, X, maybe
+from pipetools.main import pipe, X, maybe, xcurry
 from pipetools.utils import *
